@@ -7,15 +7,15 @@ use Math::GSL::Machine qw/:all/;
 use Math::GSL::Const qw/:all/;
 use Math::GSL::Errno qw/:all/;
 use Math::GSL::Vector qw/:file/;
+use Math::GSL::Test qw/:all/;
 use Carp qw/croak/;
 use Config;
 use Test::More;
 our @EXPORT = qw();
-our @EXPORT_OK = qw( ok_similar ok_status  is_similar
-                     is_similar_relative verify verify_results 
+our @EXPORT_OK = qw( 
+                     gsl_fopen gsl_fclose 
                      $GSL_MODE_DEFAULT $GSL_PREC_DOUBLE
                      $GSL_PREC_SINGLE $GSL_PREC_APPROX
-                     is_windows gsl_inf gsl_fopen gsl_fclose gsl_nan
                    );
 
 our %EXPORT_TAGS = ( 
@@ -24,7 +24,8 @@ our %EXPORT_TAGS = (
 
 our ($GSL_PREC_DOUBLE, $GSL_PREC_SINGLE, $GSL_PREC_APPROX ) = 0..2;
 our $GSL_MODE_DEFAULT = $GSL_PREC_DOUBLE;
-our $VERSION = '0.10';
+our $VERSION = '0.12';
+
 
 =head1 NAME
 
@@ -32,7 +33,7 @@ Math::GSL - Perl interface to the  GNU Scientific Library (GSL)
 
 =head1 VERSION
 
-Version 0.10
+Version 0.12
 
 =cut
 
@@ -182,8 +183,8 @@ sub new
 sub subsystems
 {
     return sort qw/
-        Diff         Machine      Statistics
-        Eigen        Matrix       Poly 
+        Diff         Machine      Statistics   
+        Eigen        Matrix       Poly      
         BSpline      Errno        PowInt        
         CBLAS        FFT          Min           IEEEUtils
         CDF          Fit          QRNG
@@ -192,124 +193,11 @@ sub subsystems
         Combination  Histogram    Multimin      Wavelet
         Complex      Histogram2D  Multiroots    Wavelet2D
         Const        Siman        Sum           Sys
-        NTuple       Integration  Sort                  
+        NTuple       Integration  Sort          Test        
         DHT          Interp       ODEIV         SF 
         Deriv        Linalg       Permutation   Spline
     /;
 }
-
-
-sub is_similar {
-    my ($x,$y, $eps, $similarity_function) = @_;
-    $eps ||= 1e-8;
-    if (ref $x eq 'ARRAY' && ref $y eq 'ARRAY') {
-        if ( $#$x != $#$y ){
-            warn "is_similar(): argument of different lengths, $#$x != $#$y !!!";
-            return 0;
-        } else {
-            map { 
-                    my $delta = abs($x->[$_] - $y->[$_]);
-                    if($delta > $eps){
-                        warn "\n\tElements start differing at index $_, delta = $delta\n";
-                        warn qq{\t\t\$x->[$_] = } . $x->[$_] . "\n";
-                        warn qq{\t\t\$y->[$_] = } . $y->[$_] . "\n";
-                        return 0;
-                    }
-                } (0..$#$x);
-            return 1;
-        }
-    } else {
-        if( ref $similarity_function eq 'CODE') {
-               $similarity_function->($x,$y) <= $eps ? return 1 : return 0;
-        } else { 
-            abs($x-$y) <= $eps ? return 1 : return 0;
-        }
-    }
-}
-sub ok_status {
-    my ($got, $expected) = @_;
-    $expected ||= $GSL_SUCCESS;
-    ok( $got == $expected, gsl_strerror(int($got)) );
-}
-sub ok_similar {
-    my ($x,$y, $msg, $eps) = @_;
-    ok(is_similar($x,$y,$eps), $msg);
-}
-
-sub is_similar_relative {
-    my ($x,$y, $eps) = @_;
-    return is_similar($x,$y,$eps, sub { abs( ($_[0] - $_[1])/abs($_[1]) ) } );
-}
-
-# this is a huge hack
-sub verify_results
-{
-    my ($results,$class) = @_;
-    # GSL uses a factor of 100 
-    my $factor   = 20; 
-    my $eps      = 2048*$Math::GSL::Machine::GSL_DBL_EPSILON; # TOL3
-    my ($x,$res);
-
-    croak "Usage: verify_results(%results, \$class)" unless $class;
-    while (my($code,$expected)=each %$results){
-        my $r        = Math::GSL::SF::gsl_sf_result_struct->new;
-        my $status   = eval qq{${class}::$code};
-
-        ok(0, qq{'$code' died} ) if !defined $status;
-        
-        if ( defined $r && $code =~ /_e\(.*\$r/) {
-            $x   = $r->{val};
-            $eps = $factor*$r->{err};
-            $res = abs($x-$expected);
-
-            if ($ENV{DEBUG} ){
-                _dump_result($r);
-                print "got $code = $x\n";
-                printf "expected   : %.18g\n", $expected ;
-                printf "difference : %.18g\n", $res;
-                printf "unexpected error of %.18g\n", $res-$eps if ($res-$eps>0);
-            }
-            if ($x =~ /nan|inf/i) {
-                    ok( $expected eq $x, "'$expected'?='$x'" );
-            } else { 
-                ok( $res <= $eps, "$code ?= $x,\nres= +-$res, eps=$eps" );    
-            }
-        }
-    }
-}
-sub verify
-{
-    my ($results,$class) = @_;
-    croak "Usage: verify(%results, \$class)" unless $class;
-    while (my($code,$result)=each %$results){
-        my $x = eval qq{${class}::$code};
-        ok(0, $@) if $@;
-
-        my ($expected,$eps);
-        if (ref $result){
-            ($expected,$eps)=@$result;
-        } else {
-            ($expected,$eps)=($result,1e-8);
-        }
-        my $res = abs($x - $expected);
-        if ($x =~ /nan|inf/i ){
-            ok( $expected eq $x, "'$expected' ?='$x'" );
-        } else {
-            ok( $res <= $eps, "$code ?= $x,\nres= +-$res, eps=$eps" );    
-        }
-    }
-}
-
-sub gsl_inf
-{ 
-    is_windows() ?  unpack 'd', scalar reverse pack 'H*', '7FF0000000000000' 
-                 : q{inf}
-}
-sub gsl_nan
-{
-    is_windows() ? unpack 'd', scalar reverse pack 'H*', '7FF8000000000000' 
-                 : q{nan}
-             }
 sub gsl_fopen
 {
     my ($file, $mode) = @_;
@@ -323,13 +211,4 @@ sub gsl_fclose
     return Math::GSL::Vector::fclose($file);
 }
 
-
-sub _dump_result($)
-{
-    my $r=shift;
-    printf "result->err: %.18g\n", $r->{err};
-    printf "result->val: %.18g\n", $r->{val};
-}
-sub is_windows() { $^O =~ /MSWin32/i }
-
 42;
